@@ -7,37 +7,29 @@ from pyhectiqlab.artifacts import SharedArtifactsManager
 from pyhectiqlab.mlmodels import download_mlmodel as ops_download_mlmodel
 from pyhectiqlab.datasets import download_dataset as ops_download_dataset
 
-def get_creds():
-	username = input("Username: ")
-	password = getpass.getpass(prompt='Password: ', stream=None) 
-	return username, password
-
 @click.group()
 def cli():
     """Just a group."""
     pass
 
 @cli.command()
-def login():
+def add_profile():
 	auth = AuthProvider()
-	if auth.is_logged():
-		click.echo("User is already logged in.")
+	username = input("Username: ")
+
+	if auth.profile_exists(username):
+		click.echo(f'A profile already exists for {username}')
 		return
 
-	username, password = get_creds()
+	password = getpass.getpass(prompt='Password: ', stream=None) 
 	click.echo("Connecting...")
-	success = auth.login_with_password(username, password)
+	success = auth.fetch_secret_api_token(username, password)
 	if success:
-		click.echo("User is authentificated.")
+		click.echo(f'Added profile [{username}] in ~/.hectiqlab/credentials')
+		click.echo("Profile added.")
 	else:
 		click.echo('Unsuccessful login.')
 
-@cli.command()
-def logout():
-	auth = AuthProvider()
-	auth.logout()
-	click.echo('Logout completed.')
-
 @cli.command()
 def version():
 	click.echo(pyhectiqlab.__version__)
