@@ -11,9 +11,18 @@ import packaging
 logger = logging.getLogger('pyhectiqlab')
 logger.setLevel(logging.WARNING)
 
+def mlmodel_info_at_path(dirpath: str):
+    try:
+        with open(os.path.join(dirpath, '.hlab_meta.json'), "r") as f:
+            data = json.load(f)
+        return data
+    except:
+        return None
+
 def upload_mlmodel(source_path: str, mlmodel_name: str, run_id: str = None, version: str = None,
     short_description: str = None, push_dir:bool = False):
     assert ' ' not in mlmodel_name, f"The name cannot contain spaces (use `{mlmodel_name.replace(' ','-')}`)"
+    assert '/' not in mlmodel_name, f"The name cannot contain / (use `{mlmodel_name.replace('/','-')}`)"
 
     manager = load_event_manager()
 
@@ -77,10 +86,16 @@ def download_mlmodel(mlmodel_name: str, project_id: str = None, version: str = N
     text += f"UUID: {meta.get('uuid')}\nDescription: {meta.get('description')}\n"
     text += f"Downloaded on: {dt.datetime.now()}"
     text += '\n--------------------------\n'
-    text += meta.get('full_description')
+    if meta.get('full_description'):
+        text += meta.get('full_description')
+    else:
+        text += 'No description.'
     with open(os.path.join(dirpath, 'HLab_README.md'), "w") as f:
         f.write(text)
 
+    with open(os.path.join(dirpath, '.hlab_meta.json'), "w") as f:
+        json.dump(meta, f)
+
     # Download the files
     files = res.get('files')
     logger.info(f'Downloading {len(files)} files')
