@@ -20,7 +20,7 @@ def lab_request(path, request_type, form_data=None, query_args=None, body_args=N
 
     headers = {}
     if token:
-        headers = {"Authorization": f"Bearer {token}"}
+        headers = {"X-API-Key": f"{token}"}
     body = None
     if body_args:
         body = json.dumps(body_args)
@@ -53,14 +53,9 @@ def lab_request(path, request_type, form_data=None, query_args=None, body_args=N
     except:
         return {"status_code": 400}
 
-def refresh_token(refresh_token):
-    res = lab_request(path="/update-token", 
-                request_type="POST", 
-                body_args=dict(refresh_token=refresh_token))
-    return res
 
-def login(username, password):
-    res = lab_request(path="/login", 
+def fetch_secret_api_token(username, password):
+    res = lab_request(path="/generate_secret_api_token", 
                 request_type="POST", 
                 form_data=dict(username=username, password=password))
     return res
@@ -153,6 +148,13 @@ def add_tag(run_id, name, description, color, token):
                      body_args=dict(tag_name=name, tag_description=description, tag_color=color, can_remove=False))
     return res
 
+def log_mlmodel(run_id, mlmodel_name, version, project_id, token):
+    res = lab_request(path=f"/runs/{run_id}/log_mlmodel",
+                     request_type="POST",
+                     token=token,
+                     query_args=dict(mlmodel_name=mlmodel_name, version=version, project_id=12))
+    return res
+
 def append_logs(run_id, logs, token):
     if logs is None:
         return
