@@ -15,9 +15,9 @@ from pyhectiqlab.mlmodels import download_mlmodel as ops_download_mlmodel
 from pyhectiqlab.mlmodels import upload_mlmodel as ops_upload_mlmodel
 from pyhectiqlab.datasets import download_dataset as ops_download_dataset
 from pyhectiqlab.datasets import upload_dataset as ops_upload_dataset
+from pyhectiqlab.mlmodels import mlmodel_info_at_path
 
-
-from functools import partial
+from functools import partial, wraps
 from sys import getsizeof
 import packaging 
 import tempfile
@@ -43,6 +43,7 @@ class Run():
             self.run_view = {"id": "null"}
             self._mock = True
             self.events_manager = EventsManager(self._mock)
+            self._read_only = False
         elif self.events_manager.is_logged()==False:
             print("User not authentificated. Use `pyhectiqlab.login()` for a web prompt or `hectiqlab login` in command line.")
             self._failed_init()
@@ -96,19 +97,19 @@ class Run():
         self.logs_buffer = Buffer(push_method=self._push_logs)
         return
 
+    def write_method(f):
+        @wraps(f)
+        def wrapper(self, *args, **kwargs):
+            if self.read_only:
+                print('Run is read only. No actions.')
+                return
+            return f(self, *args, **kwargs)
+        return wrapper
+
     @property
     def read_only(self):
         return self._read_only
 
-    def set_metrics_aggregate_method(self, aggr):
-        """
-        Set the aggregate method for the metrics manager. 
-
-        aggr [str]: One of ['none', 'sum', 'max', 'mean']
-        """
-        print('`set_metrics_aggregate_method` will be remove in a future release. Please use `set_metrics_cache_settings` instead.')
-        self.metrics_manager.set_aggr(aggr)
-
     def set_metrics_cache_settings(self, aggr: str = 'mean', max_cache_timeout=None, max_cache_length=None, min_cache_flush_delay=None):
         """
         Update the settings of the metrics manager.
@@ -131,23 +132,40 @@ class Run():
         self.events_manager = EventsManager(self._mock)
         self._read_only = True
 
+    @write_method
     def add_artifacts(self, filepath: str, step: int = None):
         """Log a file as an artifacts. If a file already exists with the name, it will be overwritten.
 
         filepath [str] : Path to the file.
         """
-        assert self._read_only==False, 'Run is in read only.'
         filename = os.path.basename(filepath)
         num_bytes = os.path.getsize(filepath)
         content_bytes = open(filepath, "rb")
         args = (self._id, filename, content_bytes, num_bytes, step)
         self.events_manager.add_event("add_file", args)
 
+    @write_method
+    def add_mlmodel_usage(self, mlmodel_name: str, version: str = None):
+        args = (self._id, mlmodel_name, version, self.project_view['id'])
+        self.events_manager.add_event("log_mlmodel", args)
+
+    @write_method
+    def add_mlmodel_usage_from_dirpath(self, dirpath:str):
+        """Log that this run is using a certain model. 
+        """
+        meta = mlmodel_info_at_path(dirpath)
+        assert meta is not None, f'Could not find the mlmodel at {dirpath}'
+
+        print(f"{meta.get('name')} - {meta.get('version')}")
+        args = (self._id, meta.get('name'), meta.get('version'), self.project_view['id'])
+        self.events_manager.add_event("log_mlmodel", args)
+        return
+        
+    @write_method        
     def add_mlmodel(self, source_path: str, name: str, version: str = None, short_description: str = None,
                 push_dir: bool = False, show_filenames: bool = False):
         """Add a mlmodel
         """
-        assert self._read_only==False, 'Run is in read only.'
         ops_upload_mlmodel(source_path=source_path, 
             mlmodel_name=name,
              run_id=self._id, 
@@ -157,19 +175,20 @@ class Run():
 
     def download_mlmodel(self, mlmodel_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
 
-        dir_path = ops_download_mlmodel(mlmodel_name=mlmodel_name, 
+        dirpath = ops_download_mlmodel(mlmodel_name=mlmodel_name, 
                         project_id=self.project_view["id"], 
                         version=version, 
                         save_path=save_path, 
                         overwrite=overwrite)
-        if dir_path is not None:
+        if dirpath is not None:
             print(dirpath) 
-
+        return dirpath
+        
+    @write_method
     def add_dataset(self, source_path: str, name: str, version: str = None, short_description: str = None,
                 push_dir: bool = False, show_filenames: bool = False):
         """Add a mlmodel
         """
-        assert self._read_only==False, 'Run is in read only.'
         ops_upload_dataset(source_path=source_path, 
             dataset_name=name,
              run_id=self._id, 
@@ -179,37 +198,38 @@ class Run():
 
     def download_dataset(self, dataset_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
 
-        dir_path = ops_download_dataset(dataset_name=dataset_name, 
+        dirpath = ops_download_dataset(dataset_name=dataset_name, 
                         project_id=self.project_view["id"], 
                         version=version, 
                         save_path=save_path, 
                         overwrite=overwrite)
-        if dir_path is not None:
+        if dirpath is not None:
             print(dirpath)        
-
+        return dirpath
+    
+    @write_method
     def add_config(self, config, prefix=""):
-        assert self._read_only==False, 'Run is in read only.'
         config.push_as_meta(self, prefix=os.path.join("config/",prefix))
 
+    @write_method
     def add_tf_model(self, model, filename, step: int = None):
-        assert self._read_only==False, 'Run is in read only.'
         save_path = tempfile.mkdtemp()
         model.save_weights(f"{save_path}/{filename}")
         p = tempfile.mkdtemp()
         shutil.make_archive(f"{p}/{filename}", 'zip', save_path)
         self.add_artifacts(f"{p}/{filename}.zip", step=step)
 
+    @write_method
     def add_directory_as_zip_artifact(self, dirpath, step: int = None):
-        assert self._read_only==False, 'Run is in read only.'
         p = tempfile.mkdtemp()
         filename = os.path.basename(dirpath)
         shutil.make_archive(f"{p}/{filename}", 'zip', root_dir=dirpath)
         self.add_artifacts(f"{p}/{filename}.zip", step=step)
 
+    @write_method
     def add_shared_artifact(self, filename):
         """Push an artifact to be shared accross the project.
         """
-        assert self._read_only==False, 'Run is in read only.'
         m = SharedArtifactsManager()
         m.push_artifact(filepath=filename, project_id=self.project_view["id"])
 
@@ -219,6 +239,7 @@ class Run():
         m = SharedArtifactsManager()
         return m.list_artifacts(project_id=self.project_view["id"])
 
+    @write_method
     def add_metrics(self, key, value, step):
         """Add a metrics. The metrics is appended even if there already exists a value
         for the same key and step. 
@@ -229,24 +250,23 @@ class Run():
         value [float or string]. If tensor, is casted to float.
         step [int]: Training step. Casted to integer.
         """
-        assert self._read_only==False, 'Run is in read only.'
         self.metrics_manager.add(key, float(value), int(step))
 
+    @write_method
     def add_tag(self, name: str, description: str = None, color: str = None):
         """Add tag to a run. If the tag does not already exists by this name,
         a new tag is created with the given description and color.
         """
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, name, description, color)
         self.events_manager.add_event("add_tag", args)
 
+    @write_method
     def add_package_versions(self, packages=None, with_sys=True, with_python=True, with_git=True):
         """Save the version of the imported packages.
 
         Use:
         add_package_versions(globals())
         """
-        assert self._read_only==False, 'Run is in read only.'
         water = Watermark()
         versions = {}
         if with_python:
@@ -263,6 +283,7 @@ class Run():
         args = (self._id, versions)
         self.events_manager.add_event("push_package_versions", args)
 
+    @write_method
     def add_package_repo_state(self, package: str = None):
         """Save the branch/commit/origin of the git repo in which
         a package is located.
@@ -270,14 +291,13 @@ class Run():
         Example:
             self.add_package_repo_state('pyhectiqlab')
         """
-        assert self._read_only==False, 'Run is in read only.'
         water = Watermark()
         result = water.get_repo_info(package)
         data = {package: result}
         args = (self._id, data)
         self.events_manager.add_event("push_git_package_state", args)
 
-
+    @write_method
     def add_current_notebook(self, stamp: str = 'datetime'):
         """If the run is executed in a jupyter notebook, this
         will quickly add your current notebook as an artifact.
@@ -285,73 +305,72 @@ class Run():
         stamp [str]: Add a stamp on the notebook name.
                     One of 'datetime', 'date', None
         """
-        assert self._read_only==False, 'Run is in read only.'
         filename = get_notebook_file(stamp)
         self.add_artifacts(filename)
 
+    @write_method
     def failed(self):
         """Set the run to the failed status.
         """
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, "failed")
         self.events_manager.add_event("set_run_status", args)
 
+    @write_method
     def _stopped(self):
         """Set the run to the stopped status.
         """
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, "stopped")
         self.events_manager.add_event("set_run_status", args)
-
+        
+    @write_method
     def completed(self):
         """Set the run to the completed with success status.
         """
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, "completed")
         self.events_manager.add_event("set_run_status", args)
-
+        
+    @write_method
     def pending(self):
         """Set the run to the pending status.
         """
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, "pending")
         self.events_manager.add_event("set_run_status", args)
-
+        
+    @write_method
     def running(self):
         """Set the run to the running status.
         """
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, "running")
         self.events_manager.add_event("set_run_status", args)
-
+        
+    @write_method
     def training(self):
         """Set the run to the training status.
         """
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, "training")
         self.events_manager.add_event("set_run_status", args)
-
+        
+    @write_method
     def add_meta(self, key, value):
         """Add or update a meta information in the format of
         key/value.
         """ 
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, key, value)
         self.events_manager.add_event("push_meta", args)
-
+        
+    @write_method
     def set_note(self, text):
         """Add or update a custom note field for the run.
         """
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, text)
         self.events_manager.add_event("set_note", args)
-
+        
+    @write_method
     def start_recording_logs(self, logger_name=None):
         """Start tailing the logs on a logger name.
         If logger_name is None, the default __name__ will be taken and
         you may use the returned logger.
         """
-        assert self._read_only==False, 'Run is in read only.'
         self.is_recording_logs = True
         self.logs_buffer.start(key="logs")
         
@@ -390,23 +409,23 @@ class Run():
             return logger
         else:
             return logging.getLogger(self.logger_name)
-
+        
+    @write_method
     def stop_recording_logs(self):
         """Stop recording the logs from the logger. 
         """
-        assert self._read_only==False, 'Run is in read only.'
         self.is_recording_logs = False
         self.logs_buffer.stop(key="logs")
-
+        
+    @write_method
     def _push_logs(self, key, msg):
-        assert self._read_only==False, 'Run is in read only.'
         if key=="logs":
             if self.is_recording_logs:
                 args = (self._id, "".join(msg))
                 self.events_manager.add_event("append_logs", args)
-
+        
+    @write_method
     def _push_metrics(self, key, values):
-        assert self._read_only==False, 'Run is in read only.'
         args = (self._id, key, values)
         self.events_manager.add_event("push_metrics", args)
 
